{*******************************************************}
{          Copyright (c) 2000 Vlad Korshunov            }
{                   IPH tools project                   }
{              Controls used in param editor            }
{*******************************************************}
unit DynamicControls;
interface
uses windows, sysutils, Classes, Controls, Session, Probes, ProbeParams,
 ExtCtrls, StdCtrls;


type
  IDynamicControl = interface
  ['{381043E0-8F51-11D4-8B2D-60834EC10000}']
    function GetTemplateItem: TTemplateItem;
    function GetParam: TParam;
    procedure SetProps(TemplateItem: TTemplateItem; Param: TParam; Value: String);
    function GetValue: String;
  end;

  TDynamicEdit = class(TEdit, IDynamicControl)
  private
    FTemplateItem: TTemplateItem;
    FParam: TParam;
  public
    function GetTemplateItem: TTemplateItem;
    function GetParam: TParam;
    procedure SetProps(TemplateItem: TTemplateItem; Param: TParam; Value: String);
    function GetValue: String;
  end;

  TDynamicShape = class(TShape, IDynamicControl)
  private
    FTemplateItem: TTemplateItem;
    FParam: TParam;
  public
    function GetTemplateItem: TTemplateItem;
    function GetParam: TParam;
    procedure SetProps(TemplateItem: TTemplateItem; Param: TParam; Value: String);
    function GetValue: String;
  end;

  TDynamicLabel = class(TLabel, IDynamicControl)
  private
    FTemplateItem: TTemplateItem;
    FParam: TParam;
  public
    function GetTemplateItem: TTemplateItem;
    function GetParam: TParam;
    procedure SetProps(TemplateItem: TTemplateItem; Param: TParam; Value: String);
    function GetValue: String;
  end;

  TDynamicCombo = class(TComboBox, IDynamicControl)
  private
    FTemplateItem: TTemplateItem;
    FParam: TParam;
  public
    function GetTemplateItem: TTemplateItem;
    function GetParam: TParam;
    procedure SetProps(TemplateItem: TTemplateItem; Param: TParam; Value: String);
    function GetValue: String;
  end;

function IDCFromControl(Control: TControl): IDynamicControl;

implementation
uses iphconsts;

function IDCFromControl(Control: TControl): IDynamicControl;
begin
  Control.GetInterface(IDynamicControl, result);
  if result = nil then
    Raise Exception.Create(sDynamicControlsOnly);
end;

{ TDynamicEdit }

function TDynamicEdit.GetParam: TParam;
begin
  result:=FParam;
end;

function TDynamicEdit.GetTemplateItem: TTemplateItem;
begin
  result:=FTemplateItem;
end;

function TDynamicEdit.GetValue: String;
begin
  result:=Text;
end;

procedure TDynamicEdit.SetProps(TemplateItem: TTemplateItem; Param: TParam; Value: String);
begin
  FParam:=Param;
  FTemplateItem:=TemplateItem;
  Text:=Value;
end;

{ TDynamicShape }

function TDynamicShape.GetParam: TParam;
begin
  result:=FParam;
end;

function TDynamicShape.GetTemplateItem: TTemplateItem;
begin
  result:=FTemplateItem;
end;

function TDynamicShape.GetValue: String;
begin
  result:='';
end;

procedure TDynamicShape.SetProps(TemplateItem: TTemplateItem; Param: TParam; Value: String);
begin
  FParam:=Param;
  FTemplateItem:=TemplateItem;
end;

{ TDynamicLabel }

function TDynamicLabel.GetParam: TParam;
begin
  result:=FParam;
end;

function TDynamicLabel.GetTemplateItem: TTemplateItem;
begin
  result:=FTemplateItem;
end;

function TDynamicLabel.GetValue: String;
begin
  result:=Caption;
end;

procedure TDynamicLabel.SetProps(TemplateItem: TTemplateItem; Param: TParam; Value: String);
begin
  FParam:=Param;
  FTemplateItem:=TemplateItem;
  Caption:=Value;
end;

{ TDynamicCombo }

function TDynamicCombo.GetParam: TParam;
begin
  result:=FParam;
end;

function TDynamicCombo.GetTemplateItem: TTemplateItem;
begin
  result:=FTemplateItem;
end;

function TDynamicCombo.GetValue: String;
begin
  result:=Text;
end;

procedure TDynamicCombo.SetProps(TemplateItem: TTemplateItem;
  Param: TParam; Value: String);
begin
  FParam:=Param;
  FTemplateItem:=TemplateItem;
  Text:=Value;
end;

end.
