const
//ip status values 
  ip_status_base           = 11000;
  ip_success               = 0;
  ip_buf_too_small         = ip_status_base +  1;
  ip_dest_net_unreachable  = ip_status_base +  2;
  ip_dest_host_unreachable = ip_status_base +  3;
  ip_dest_prot_unreachable = ip_status_base +  4;
  ip_dest_port_unreachable = ip_status_base +  5;
  ip_no_resources          = ip_status_base +  6;
  ip_bad_options           = ip_status_base +  7;
  ip_hw_error              = ip_status_base +  8;
  ip_packet_too_big        = ip_status_base +  9;
  ip_req_timed_out         = ip_status_base + 10;
  ip_bad_req               = ip_status_base + 11;
  ip_bad_route             = ip_status_base + 12;
  ip_ttl_expired_transmit  = ip_status_base + 13;
  ip_ttl_expired_reassem   = ip_status_base + 14;
  ip_param_problem         = ip_status_base + 15;
  ip_source_quench         = ip_status_base + 16;
  ip_option_too_big        = ip_status_base + 17;
  ip_bad_destination       = ip_status_base + 18;
  ip_addr_deleted          = ip_status_base + 19;
  ip_sepc_mtu_change       = ip_status_base + 20;
  ip_mtu_change            = ip_status_base + 21;
  ip_unload                = ip_status_base + 22;
  ip_general_failure       = ip_status_base + 50;
  max_ip_status            = ip_general_failure;
  ip_pending               = ip_status_base +255;

//ICMP types
  ICMP_ECHOREPLY           = 0  ;  (* echo reply *)
  ICMP_UNREACH             = 3  ;  (* dest unreachable, codes: *)
  ICMP_UNREACH_NET         = 0  ;  (* bad net *)
  ICMP_UNREACH_HOST        = 1  ;  (* bad host *)
  ICMP_UNREACH_PROTOCOL    = 2  ;  (* bad protocol *)
  ICMP_UNREACH_PORT        = 3  ;  (* bad port *)
  ICMP_UNREACH_NEEDFRAG    = 4  ;  (* IP_DF caused drop *)
  ICMP_UNREACH_SRCFAIL     = 5  ;  (* src route failed *)
  ICMP_SOURCEQUENCH        = 4  ;  (* packet lost, slow down *)
  ICMP_REDIRECT            = 5  ;  (* shorter route, codes: *)
  ICMP_REDIRECT_NET        = 0  ;  (* for network *)
  ICMP_REDIRECT_HOST       = 1  ;  (* for host *)
  ICMP_REDIRECT_TOSNET     = 2  ;  (* for tos and net *)
  ICMP_REDIRECT_TOSHOST    = 3  ;  (* for tos and host *)
  ICMP_ECHO                = 8  ;  (* echo service *)
  ICMP_TIMXCEED            = 11 ;  (* time exceeded, code: *)
  ICMP_TIMXCEED_INTRANS    = 0  ;  (* ttl==0 in transit *)
  ICMP_TIMXCEED_REASS      = 1  ;  (* ttl==0 in reass *)
  ICMP_PARAMPROB           = 12 ;  (* ip header bad *)
  ICMP_TSTAMP              = 13 ;  (* timestamp request *)
  ICMP_TSTAMPREPLY         = 14 ;  (* timestamp reply *)
  ICMP_IREQ                = 15 ;  (* information request *)
  ICMP_IREQREPLY           = 16 ;  (* information reply *)
  ICMP_MASKREQ             = 17 ;  (* address mask request *)
  ICMP_MASKREPLY           = 18 ;  (* address mask reply *)

//Options for use with [gs]etsockopt at the IP level. (corrected from winsock)
  IP_TTL              =  4;           { Time to live of IP packet }
  IP_MULTICAST_IF     =  9;           { set/get IP multicast interface   }
  IP_MULTICAST_TTL    = 10;           { set/get IP multicast timetolive  }
  IP_MULTICAST_LOOP   = 11;           { set/get IP multicast loopback    }
  IP_ADD_MEMBERSHIP   = 12;           { add  an IP group membership      }
  IP_DROP_MEMBERSHIP  = 13;           { drop an IP group membership      }

type
  p_ip_options=^t_ip_options;
  t_ip_options=packed record
    Ttl: byte;             (* time to live *)
    Tos: byte;             (* type of service (usually 0) *)
    flags: byte;           (* IP header flags (usually 0) *)
    optionssize: byte;     (* size of options data (usually 0, max 40) *)
    optionsdata: pointer;  (* options data buffer *)
  end;

  t_ip_header=packed record
    ip_hl_v: byte;   (* low nibble: header length; high nibble: version *)
    ip_tos: byte;    (* type of service *)
    ip_len: word;    (* total length *)
    ip_id: word;     (* identification *)
    ip_off: word;    (* fragment offset field *)
    ip_ttl: byte;    (* time to live *)
    ip_p: byte;      (* protocol *)
    ip_sum: word;    (* checksum *)
    ip_src,
    ip_dst:longint;  (* source and dest address *)
  end;

  t_icmp_echo_request=packed record
    icmp_type: byte;            (* ICMP type *)
    icmp_code: byte;            (* ICMP code *)
    icmp_cksum: word;           (* ICMP checksum *)
    icmp_id: word;              (* ICMP identification *)
    icmp_seq: word;             (* ICMP sequence number *)
  end;

  t_icmp_reply=packed record
    icmp_type: byte;            (* ICMP type *)
    icmp_code: byte;            (* ICMP code *)
    icmp_cksum: word;           (* ICMP checksum *)
    icmp_unused: longint;       (* unused area *)
    icmp_ip: t_ip_header;       (* original IP header which cause the reply *)
    icmp_dgram:array[0..63] of byte;   (* first 64 bits of datagram *)
  end;

  p_icmp_echo_reply=^t_icmp_echo_reply;
  t_icmp_echo_reply=packed record
    address: u_long;           (* source address *)
    status: u_long;            (* IP status *)
    rttime: u_long;            (* rount trip time in milliseconds *)
    datasize: word;            (* reply data size *)
    reserved: word;            (* who knows *)
    data: pointer;             (* reply data buffer *)
    ip_options:t_ip_options;   (* reply options *)
  end;

  TICMPreply_buf=record
    ip_header  : t_ip_header;
    icmp_header: t_icmp_echo_request;
    time       : longint;
  end;

  TICMP_sendblock=record
    icmp_header: t_icmp_echo_request;
    time       : longint;
    data       : char;               (* to be extended dynamically *)
  end;


