{*******************************************************}
{          Copyright (c) 2000 Vlad Korshunov            }
{                   IPH tools project                   }
{                Thread safe logger unit                }
{*******************************************************}
unit LogFile;
interface
uses windows, sysutils;

procedure LogError(s: String);

implementation
const
  LogHandle: DWord = 0;
  sLogName = 'c:\IPHTools.log';
var
  hLogFileMutex: THandle;


const
  CMonths = 'JanFebMarAprMayJunJulAugSepOctNovDec';
  Months: string[Length(CMonths)] = CMonths;
  CDays='SunMonTueWedThuFriSat';
  Days: string[Length(CDays)] = CDays;

  //   -   
  WriteThrough: Boolean = True;

function MonthE(m: Integer): string;
begin
  Result := Copy(Months, 1+(m-1)*3, 3);
end;

function SysTimeToStr(lt: TSystemTime): string;
begin
  With lt do result:=Format('%2d %s %d %2d:%2d:%2d',
    [wDay, MonthE(wMonth), wYear, wHour, wMinute, wSecond])
end;


function CurTime: string;
var
  lt: TSystemTime;
begin
  GetLocalTime(lt);
  result:=SysTimeToStr(lt);
end;

{   c  ,   - }
function OpenLog(var Handle: DWORD): Boolean;
var
  Flags: dword;
begin
  if Handle = 0 then
  begin
    if WriteThrough then Flags:=FILE_FLAG_WRITE_THROUGH else Flags:=0;
    Handle := CreateFile(sLogName, GENERIC_WRITE, FILE_SHARE_READ+FILE_SHARE_WRITE, Nil,
              OPEN_ALWAYS, Flags, 0);
    if Handle = INVALID_HANDLE_VALUE then
      MessageBox(hInstance, PChar('Unable to open debug file. '+SysErrorMessage(GetLastError)), 'CreateFile error', 0);
  end;
  Result := Handle <> INVALID_HANDLE_VALUE;
end;

procedure LogError(s: String);
var
  b: dword;
Function WaitMutex: Boolean;
var
  dwWaitResult: DWORD;
begin
  Result:=False;
  dwWaitResult := WaitForSingleObject(hLogFileMutex,1000);// one-second time-out interval
  Case dwWaitResult of
    WAIT_OBJECT_0: Result:=True;
    WAIT_TIMEOUT: MessageBox(hInstance, PChar('Unable to write to log file.'+SysErrorMessage(GetLastError)), 'Mutex wait time-out', 0);
    WAIT_ABANDONED: MessageBox(hInstance, PChar('Unable to write to log file.'+SysErrorMessage(GetLastError)), 'Mutex abandoned', 0);
  end;
end;

begin
  s := Format('[%10x][%s] %s'+#13#10, [GetCurrentProcessID, CurTime, s]);
  if WaitMutex then
  begin
    if not OpenLog(LogHandle) then Halt;
    if SetFilePointer(LogHandle, 0, nil, FILE_END) = INVALID_FILE_SIZE then
    begin
      MessageBox(hInstance, PChar('Unable to write to log file. '+SysErrorMessage(GetLastError)), 'SetFilePointer error', 0);
      CloseHandle(LogHandle);
    end;
    WriteFile(LogHandle, s[1], Length(s), b, nil);
    ReleaseMutex(hLogFileMutex);
  end;
end;

initialization
  hLogFileMutex := CreateMutex(nil, false, 'iphtools.debugmutex.');
  if hLogFileMutex=0 then
    MessageBox(hInstance, PChar('Unable to create log mutex.'+SysErrorMessage(GetLastError)), 'Create mutex', 0);
end.
