unit FileIterator;
interface
uses windows, sysutils, classes, Plugin, Probes, ProbeParams;

Type
  TFileIterator = class(TIteratorProbe)
    FStrings: TStrings;
    FCurLine: Integer;
  protected
    procedure CheckRequiredParams; override;
    procedure AddSupportedParams(var Params: TParams); override;
    function InternalAction: Boolean; override;
    function NextAction: Boolean; override;
  public
    class function GetName: string; override;
    class function GetDescription: String; override;
    destructor Destroy; override;
  end;

implementation

{ TFileIterator }
procedure TFileIterator.AddSupportedParams(var Params: TParams);
begin
  inherited;
  Params.CreateParam(ftFileName, 'FileName', ptInput, [poRequired]);
  Params.CreateParam(ftString, 'FileLine', ptOutput, []);
end;

procedure TFileIterator.CheckRequiredParams;
begin
  inherited;
  if Not FileExists(Params.ParamByName('FileName').AsString) then
    Raise Exception.CreateFmt('File ''%s''not found',
      [Params.ParamByName('FileName').AsString]);
end;

destructor TFileIterator.Destroy;
begin
  if FStrings<>nil then FStrings.Free;
  inherited;
end;

class function TFileIterator.GetDescription: String;
begin
  result:='Iterate lines in text file';
end;

class function TFileIterator.GetName: string;
begin
  result:='Text file iterator';
end;

function TFileIterator.InternalAction: Boolean;
var
  i: integer;
begin
  Result:=True;
  FStrings:=TStringList.Create;
  FStrings.LoadFromFile(Params.ParamByName('FileName').AsString);
  for i:=FStrings.Count-1 to 0 do
    if FStrings[i] = '' then FStrings.Delete(i);
  FCurLine:=0;
  if FStrings.Count=0 then
  begin
    DoOnError(0, 'Empty file');
    Result:=False;
  end;
end;

function TFileIterator.NextAction: Boolean;
begin
  Result:=False;
  if FCurLine<=FStrings.Count-1 then
  begin
    Params.ParamByName('FileLine').AsString:=Trim(FStrings[FCurLine]);
    Result:=not (FCurLine = FStrings.Count-1);
    DoOnProbe(not Result);
    inc(FCurLine);          
  end;
end;

initialization
  RegisterProbes([TFileIterator]);
end.
