{*******************************************************}
{          Copyright (c) 2000 Vlad Korshunov            }
{                   IPH tools project                   }
{                    Session window                     }
{*******************************************************}
unit SessionWindow;

interface

uses Windows, Messages, Classes, graphics, sysutils, Forms, Controls, StdCtrls, ComCtrls,
  ActnList, Menus, winsock, Placemnt, Dialogs, probes, ChildWin, Session, ProbeParams,
  ImgList, ToolWin, ExtCtrls, TemplateEditor;

const
  WM_FreeSession = WM_USER+100;

type
  TfmSessionWindow = class(TfmChildWindow)
    ActionList1: TActionList;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    acStartSession: TAction;
    ScanIPRange1: TMenuItem;
    StatusBar1: TStatusBar;
    OpenDialog: TOpenDialog;
    FormStorage1: TFormStorage;
    pnTop: TPanel;
    pnList: TPanel;
    Label2: TLabel;
    Splitter1: TSplitter;
    pnMemo: TPanel;
    Label3: TLabel;
    ToolBar1: TToolBar;
    Images: TImageList;
    ToolButton1: TToolButton;
    acTerminateSession: TAction;
    ToolButton2: TToolButton;
    acTerminateSession1: TMenuItem;
    lvHosts: TListView;
    lvHostResults: TListView;
    acSortView: TAction;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    acShowOnlySuccess: TAction;
    ToolButton5: TToolButton;
    acShowAll: TAction;
    ToolButton6: TToolButton;
    acEditView: TAction;
    acEditTemplate: TAction;
    ToolButton7: TToolButton;
    ToolButton8: TToolButton;
    ToolButton9: TToolButton;
    procedure acStartSessionExecute(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure acTerminateSessionExecute(Sender: TObject);
    procedure lvHostsSelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure lvHostResultsResize(Sender: TObject);
    procedure lvHostsCompare(Sender: TObject; Item1, Item2: TListItem;
      Data: Integer; var Compare: Integer);
    procedure acSortViewExecute(Sender: TObject);
    procedure acShowOnlySuccessExecute(Sender: TObject);
    procedure acShowAllExecute(Sender: TObject);
    procedure acEditViewExecute(Sender: TObject);
    procedure lvHostResultsCustomDrawItem(Sender: TCustomListView;
      Item: TListItem; State: TCustomDrawState; var DefaultDraw: Boolean);
    procedure acEditTemplateExecute(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
//    procedure cdsHostsCalcFields(DataSet: TDataSet);
  private
    FTemplate: TSessionTemplate;
    FSession: TSession;
    FResults: TSessionResults;
    FShowErrorResults: Boolean;
    FShowAllResults: Boolean;
    FMaxNo: Integer;
    fmTemplateEditor: TfmTemplateEditor;
    procedure UpdateStatus;
    procedure SetProbesParams(Session: TSession; var Continue: Boolean);
    procedure ProbeStateChange(Session: TSession; SessionProbe: TSessionProbe);
    procedure SessionTerminate(Sender: TObject);
    procedure SessionException(Session: TSession; E: Exception);
    procedure WMFreeSession(var Message: TMessage); message WM_FreeSession;
    procedure WMChildclosed(var Message: TMessage); message WM_Child_Closed;
    procedure AddResult(Result: TSessionResult);
    function FindResultInView(GroupValues: TStrings): TListItem;
    function NewViewItem(GroupValues: TStrings): TListItem;
    procedure UpdateResults(Item: TListItem);
    procedure ClearViews;
    procedure TemplateItemRemoved(Item: TTemplateItem);
    function CreateTemplate: TSessionTemplate;
  public
    procedure LoadFromFile(const FileName: string); override;
    procedure SaveToFile(const FileName: string); override;
    function DefaultExtension: string; override;
    procedure NewFile; override;
    procedure PrepareView;
    procedure RebuildView;

  end;

implementation

uses iphconsts, utils, contutils, ParamEditor, ViewEditor;

const
  itBackGrayed = $01;
  itErrorItem = $02;

{$R *.DFM}

procedure TfmSessionWindow.acStartSessionExecute(Sender: TObject);
begin
  if FResults.Count<>0 then
  case MessageDlg(sClearPrevious, mtConfirmation, mbYesNoCancel, 0) of
    mrYes:
      begin
        FResults.FreeItems;
        FResults.Clear;
        ClearViews;
      end;
    mrCancel: exit;
  end;
  FSession:=TSession.create(FTemplate);
  FSession.OnSetProbesParams:=SetProbesParams;
  FSession.OnProbeStateChange:=ProbeStateChange;
  FSession.OnTerminate:=SessionTerminate;
  FSession.OnException:=SessionException;
  FSession.Resume;
end;

procedure TfmSessionWindow.FormCreate(Sender: TObject);
begin
  FResults:=TSessionResults.Create;
end;

procedure TfmSessionWindow.FormDestroy(Sender: TObject);
begin
  acTerminateSession.Execute;
  if FTemplate<>nil then FTemplate.Free;
  FResults.Free;
end;

procedure TfmSessionWindow.UpdateStatus;
begin
  if (FSession<>nil) then
  begin
    acStartSession.Enabled:=False;
    acTerminateSession.Enabled:=True;
    acEditView.Enabled:=False;
    acEditTemplate.Enabled:=False;
    Statusbar1.Panels[0].Text:=sStatusActive;
    Statusbar1.Panels[1].Text:=Format(sProbesCount, [FSession.ActiveProbeCount]);
  end else
  begin
    Statusbar1.Panels[0].Text:=sStatusInactive;
    Statusbar1.Panels[1].Text:='';
    acStartSession.Enabled:=True;
    acTerminateSession.Enabled:=False;
    acEditView.Enabled:=True;
    acEditTemplate.Enabled:=True;
  end;
end;

function TfmSessionWindow.DefaultExtension: string;
begin
  result:=extSession;
end;

procedure TfmSessionWindow.NewFile;
begin
  inherited;
  OpenDialog.Filter:=Format('%s|%s', [GetFilter(extTemplate), GetFilter(extAllFiles)]);
  OpenDialog.InitialDir:=ExtractFilePath(ParamStr(0));
  FTemplate:=CreateTemplate;
  if OpenDialog.Execute then
  begin
    FTemplate.LoadFromFile(OpenDialog.FileName);
    PrepareView;
  end;
end;

procedure TfmSessionWindow.SetProbesParams(Session: TSession; var Continue: Boolean);
begin
  Continue:=EditParams(Session.Template, True);
end;

procedure TfmSessionWindow.ProbeStateChange(Session: TSession;
  SessionProbe: TSessionProbe);
begin
  if not SessionProbe.Active then
    AddResult(FResults.NewItem(SessionProbe.TemplateItem, SessionProbe.Probe.Params,
      SessionProbe.Success));
  DataChanged:=true;
  UpdateStatus;
end;

procedure TfmSessionWindow.SessionTerminate(Sender: TObject);
begin
  PostMessage(Self.Handle, WM_FreeSession, 0, 0);
  UpdateStatus;
end;

procedure TfmSessionWindow.FormShow(Sender: TObject);
begin
  inherited;
  UpdateStatus;
end;

procedure TfmSessionWindow.WMFreeSession(var Message: TMessage);
begin
  FSession.Free;
  FSession:=nil;
  UpdateStatus;
end;

procedure TfmSessionWindow.acTerminateSessionExecute(Sender: TObject);
begin
  if (FSession<>nil) then FSession.Terminate;
end;

procedure TfmSessionWindow.SessionException(Session: TSession;
  E: Exception);
begin
  Application.ShowException(E);
end;

procedure TfmSessionWindow.PrepareView;
var
  i: Integer;
  Column: TListColumn;
  FirstColWidth: Integer;
begin
  Assert(FTemplate<>nil);
  lvHosts.Columns.Clear;
  if not FShowAllResults then
  begin
    Column:=lvHosts.Columns.Add;
    Column.Caption:=sResultNumbColumn;
    FirstColWidth:=Column.Width;
    for i:=0 to FTemplate.View.Groups.Count-1 do
    begin
      if FTemplate.View.Groups[i].GroupName <> grVisibleParams then
      begin
        Column:=lvHosts.Columns.Add;
        Column.Caption:=FTemplate.View.Groups[i].GroupName;
        Column.Width:=((lvHosts.Width-FirstColWidth) div (FTemplate.View.Groups.Count-1))-5;
      end;
    end;
  end else
  begin
    Column:=lvHosts.Columns.Add;
    Column.Caption:='??';
  end;
end;

procedure TfmSessionWindow.AddResult(Result: TSessionResult);
var
  i, j: Integer;
  TemplateParam: TParam;
  GroupValues: TStrings;
  Item: TListItem;
  NotEmpty: boolean;
begin
  if not Result.Success and not FShowErrorResults then Exit;
  NotEmpty:=False;
  GroupValues:=TStringList.Create;
  try
    for i:=0 to FTemplate.View.Groups.Count-1 do
    begin
      if FTemplate.View.Groups[i].GroupName <> grVisibleParams then
      begin
        GroupValues.AddObject('', nil);
        for j:=0 to Result.Params.Count-1 do
        begin
          TemplateParam:=Result.TemplateItem.Params.FindParam(Result.Params[j].Name);
          // Is param in group ?
          if FTemplate.View.Groups[i].IndexOf(TemplateParam)<>-1 then
          begin
            GroupValues[i-1]:=Result.Params[j].AsString;
            GroupValues.Objects[i-1]:=Pointer(1);
            if GroupValues[i-1]<>'' then NotEmpty:=True;
          end;
        end;
      end;
    end;
    if NotEmpty then
    begin
      Item:=FindResultInView(GroupValues);
      if Item = nil then
      begin
        Item:=NewViewItem(GroupValues);
        Result.OwnerData:=Item;
      end else
      begin
        for i:=0 to GroupValues.Count-1 do
          if Item.SubItems[i]='' then Item.SubItems[i]:=GroupValues[i];
        Result.OwnerData:=Item;
        if lvHosts.Selected=Item then UpdateResults(Item);
      end;
    end;
  finally
    GroupValues.Free;
  end;
end;

function TfmSessionWindow.FindResultInView(
  GroupValues: TStrings): TListItem;

  function SameValue(Item: TListItem; Index: Integer; const Value: String): Boolean;
  begin
    result:=(Item.SubItems[Index]=Value);
  end;

var
  i, j, Count: integer;
begin
  result:=nil;
  for i:=0 to lvHosts.Items.Count-1 do
  begin
    Count:=0;
    for j:=0 to GroupValues.Count-1 do
      if (GroupValues.Objects[j]=nil) or
        SameValue(lvHosts.Items[i], j, GroupValues[j]) then inc(Count);
    if Count = GroupValues.Count  then
    begin
      result:=lvHosts.Items[i];
      break;
    end;
  end;
end;

function TfmSessionWindow.NewViewItem(GroupValues: TStrings): TListItem;
var
  i: integer;
begin
  inc(FMaxNo);
  result:=lvHosts.Items.Add;
  result.Caption:=IntToStr(FMaxNo);
  for i:=0 to GroupValues.Count-1 do
    result.SubItems.Add(GroupValues[i]);
end;

procedure TfmSessionWindow.lvHostsSelectItem(Sender: TObject;
  Item: TListItem; Selected: Boolean);
begin
  if Selected then
    if FShowAllResults then UpdateResults(nil) else UpdateResults(Item);
end;

procedure TfmSessionWindow.UpdateResults(Item: TListItem);
var
  ItemStyle: Integer;

  function ShowThis(SessionResult: TSessionResult): boolean;
  begin
    Result:=SessionResult.Success or FShowErrorResults;
  end;

  procedure AddResultInfo(Result: TSessionResult);
  var
    NewItem: TListItem;
  begin
    NewItem:=lvHostResults.Items.Add;
    NewItem.Caption:=Result.TemplateItem.Alias;
    NewItem.SubItems.Add(DateTimeToStr(Result.Time)+
      ' ('+SuccessStr(Result.Success)+')');
    NewItem.Data:=Pointer(ItemStyle);
    SetImageIndex(NewItem, 4);
  end;

  procedure AddParam(Param: TParam);
  var
    NewItem: TListItem;
  begin
    if (Param.Name <> tkResultTimeStamp) and (Param.Name<>tkResultSuccess) and
      (Param.AsString<>'') then
    begin
      NewItem:=lvHostResults.Items.Add;
      NewItem.Caption:=Param.Name;
      NewItem.SubItems.Add(Param.AsString);
      if Param.ParamType = ptInput then SetImageIndex(NewItem, 2) else SetImageIndex(NewItem, 3);
      NewItem.Data:=Pointer(ItemStyle);
    end;
  end;

var
  i, j: integer;

  Param: TParam;
begin
  ItemStyle:=0;
  lvHostResults.Items.BeginUpdate;
  try
    lvHostResults.Items.Clear;
    for i:=0 to FResults.Count-1 do
    begin
      if ((Item = nil) or (FResults[i].OwnerData = Item)) and ShowThis(FResults[i]) then
      begin
        if FResults[i].Success then
          ItemStyle:=ItemStyle and not itErrorItem else
          ItemStyle:=ItemStyle or itErrorItem;
        AddResultInfo(FResults[i]);
        if FShowAllResults then
          for j:=0 to FResults[i].Params.Count-1 do
            AddParam(FResults[i].Params[j])
        else
          for j:=0 to FTemplate.View.Groups[0].Count-1 do
            if FResults[i].TemplateItem = GetParamOwner(FTemplate.View.Groups[0][j]) then
            begin
              Param:=FResults[i].Params.FindParam(FTemplate.View.Groups[0][j].Name);
              if Param<>nil then AddParam(Param);
            end;
        ItemStyle:=ItemStyle xor itBackGrayed;
      end;
    end;
  finally
    lvHostResults.Items.EndUpdate;
  end;
end;

procedure TfmSessionWindow.lvHostResultsResize(Sender: TObject);
var
  MaxWidth: Integer;
begin
  MaxWidth:=lvHostResults.Width-lvHostResults.Columns[0].Width-5;
  if lvHostResults.Columns[1].Width<MaxWidth then 
    lvHostResults.Columns[1].Width:=MaxWidth;
end;

procedure TfmSessionWindow.lvHostsCompare(Sender: TObject; Item1,
  Item2: TListItem; Data: Integer; var Compare: Integer);
var
  i: integer;
begin
  if (Item1<>nil) and (Item2<>nil) then
  begin
    for i:=0 to Item1.SubItems.Count-1 do
    begin
      Compare:=CompareStr(Item1.SubItems[i], Item2.SubItems[i]);
      if Compare<>0 then break;
    end;
  end;
end;

procedure TfmSessionWindow.acSortViewExecute(Sender: TObject);
begin
  lvHosts.AlphaSort;
end;

procedure TfmSessionWindow.acShowOnlySuccessExecute(Sender: TObject);
begin
  FShowErrorResults:=Not FShowErrorResults;
  RebuildView;
end;

procedure TfmSessionWindow.RebuildView;
var
  i: integer;
  Item: TListItem;
begin
  ClearViews;
  PrepareView;
  if FShowAllResults then
  begin
    Item:=lvHosts.Items.Add;
    Item.Caption:=sResultAllResults;
    UpdateResults(nil);
  end else
  begin
    for i:=0 to FResults.Count-1 do
    begin
      FResults[i].OwnerData:=nil;
      AddResult(FResults[i]);
    end;
    lvHosts.AlphaSort;
  end;
end;

procedure TfmSessionWindow.ClearViews;
begin
  FMaxNo:=0;
  lvHosts.Items.BeginUpdate;
  lvHostResults.Items.BeginUpdate;
  try
    lvHosts.Items.Clear;
    lvHostResults.Items.Clear;
  finally
    lvHosts.Items.EndUpdate;
    lvHostResults.Items.EndUpdate;
  end;
end;

procedure TfmSessionWindow.acShowAllExecute(Sender: TObject);
begin
  FShowAllResults:=not FShowAllResults;
  RebuildView;
end;

procedure TfmSessionWindow.acEditViewExecute(Sender: TObject);
begin
  EditView(FTemplate);
  DataChanged:=True;
  RebuildView;
end;

procedure TfmSessionWindow.lvHostResultsCustomDrawItem(
  Sender: TCustomListView; Item: TListItem; State: TCustomDrawState;
  var DefaultDraw: Boolean);
begin
  if Item = nil then Exit;
  if (Integer(Item.Data) and itBackGrayed) <> 0 then
    lvHostResults.Canvas.Brush.Color:=clActiveBorder else
    lvHostResults.Canvas.Brush.Color:=clWindow;
  if (Integer(Item.Data) and itErrorItem) <> 0 then
    lvHostResults.Canvas.Font.Color:=clRed else
    lvHostResults.Canvas.Font.Color:=clWindowText;
end;

procedure TfmSessionWindow.LoadFromFile(const FileName: string);
var
  Stream: TFileStream;
begin
  FTemplate:=CreateTemplate;
  FResults:=TSessionResults.Create;
  Stream:=TFileStream.Create(FileName, fmOpenRead or fmShareDenyWrite);
  try
    with TSessionParser.Create(Stream, FTemplate, FResults) do
    try
      Parse;
    finally
      Free;
    end;
  finally
    Stream.Free;
  end;
  DataChanged:=False;
  RebuildView;  
end;

procedure TfmSessionWindow.SaveToFile(const FileName: string);
var
  Stream: TFileStream;
begin
  Stream:=TFileStream.Create(FileName, fmCreate);
  try
    FTemplate.SaveToStream(Stream);
    FResults.SaveToStream(Stream);
  finally
    Stream.Free;
  end;
  DataChanged:=False;
end;

procedure TfmSessionWindow.acEditTemplateExecute(Sender: TObject);
begin
  if fmTemplateEditor=nil then
  begin
    fmTemplateEditor := TfmTemplateEditor.Create(Application);
    fmTemplateEditor.Caption:=Self.Caption+sTemplate;
    fmTemplateEditor.OwnerForm:=Self;
    fmTemplateEditor.Template:=fTemplate;
  end else
    fmTemplateEditor.BringToFront;
end;

procedure TfmSessionWindow.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  inherited;
  if fmTemplateEditor<>nil then
  begin
    fmTemplateEditor.Release;
    fmTemplateEditor:=nil;
  end;
end;

procedure TfmSessionWindow.WMChildclosed(var Message: TMessage);
begin
  if Pointer(Message.WParam) = fmTemplateEditor then
  begin
    fmTemplateEditor:=nil;
    RebuildView;
  end;
end;

function TfmSessionWindow.CreateTemplate: TSessionTemplate;
begin
  result:=TSessionTemplate.Create;
  Result.OnItemRemoved:=TemplateItemRemoved;
end;

procedure TfmSessionWindow.TemplateItemRemoved(Item: TTemplateItem);
var
  i: integer;
  Result: TSessionResult;
begin
  if not (csDestroying in ComponentState) then
  begin
    // Remove results for this item
    for i:=FResults.Count-1 downto 0 do
      if FResults[i].TemplateItem = Item then
      begin
        Result:=FResults[i];
        FResults.Remove(Result);
        Result.Free;
      end;
    DataChanged:=True;
    RebuildView;
  end;
end;

end.
