{*******************************************************}
{          Copyright (c) 2000 Vlad Korshunov            }
{                   IPH tools project                   }
{                 Visual template editor                }
{*******************************************************}
unit TemplateEditor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ComCtrls, childwin, session, ExtCtrls, Placemnt,
  ActnList, ImgList, ToolWin, TemplateFrame;

type
  TfmTemplateEditor = class(TfmChildWindow)
    pnProbes: TGroupBox;
    lvProbes: TListView;
    pnComment: TGroupBox;
    Memo1: TMemo;
    Splitter1: TSplitter;
    FormStorage1: TFormStorage;
    ToolBar1: TToolBar;
    ActionList1: TActionList;
    ToolButton1: TToolButton;
    acShowInputParams: TAction;
    acShowOutputParams: TAction;
    ToolButton2: TToolButton;
    acEditParams: TAction;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    frTemplate: TfrTemplate;
    acEditView: TAction;
    ToolButton5: TToolButton;
    pnDescription: TGroupBox;
    Memo2: TMemo;
    Splitter2: TSplitter;
    Splitter3: TSplitter;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure lvProbesMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure tvTemplateDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure tvTemplateDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure tvTemplateMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure lvProbesDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure lvProbesDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure tvTemplateEdited(Sender: TObject; Node: TTreeNode;
      var S: String);
    procedure acShowInputParamsExecute(Sender: TObject);
    procedure acShowOutputParamsExecute(Sender: TObject);
    procedure tvTemplateEditing(Sender: TObject; Node: TTreeNode;
      var AllowEdit: Boolean);
    procedure acEditParamsExecute(Sender: TObject);
    procedure acEditViewExecute(Sender: TObject);
    procedure lvProbesSelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure frTemplatetvTemplateMouseUp(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure frTemplatetvTemplateMouseMove(Sender: TObject;
      Shift: TShiftState; X, Y: Integer);
    procedure lvProbesMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure lvProbesMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
  private
    FTemplate: TSessionTemplate;
    FDraggedItem: TListItem;
    FDraggedNode: TTreeNode;
    FOwnerForm: TForm;
    FDragStartX: Integer;
    FDragStartY: Integer;
    procedure ClearProbeView;
    procedure ProbeListToView;
    procedure SetTemplate(const Value: TSessionTemplate);
  public
    procedure LoadFromFile(const FileName: string); override;
    procedure SaveToFile(const FileName: string); override;
    function DefaultExtension: string; override;
    property Template: TSessionTemplate read FTemplate write SetTemplate;
    property OwnerForm: TForm read FOwnerForm write FOwnerForm;
  end;

var
  fmTemplateEditor: TfmTemplateEditor;

implementation
uses probes, probeparams, ParamEditor, ViewEditor;
const
  TreshHold = 5;

{$R *.DFM}

{ TfmTemplateEditor }
procedure TfmTemplateEditor.ClearProbeView;
var
  i: integer;
begin
  lvProbes.Items.BeginUpdate;
  try
    for i:=0 to lvProbes.Items.Count-1 do
    begin
      if lvProbes.Items[i].Data<>nil then
        TCustomProbe(lvProbes.Items[i].Data).Free;
    end;
    lvProbes.Items.Clear;
  finally
    lvProbes.Items.EndUpdate;
  end;
end;

procedure TfmTemplateEditor.ProbeListToView;
var
  ProbeList: TStrings;
  i: integer;
  Item: TListItem;
  Selected: Boolean;
begin
  ProbeList:=TStringList.Create;
  GetAvailableProbes(ProbeList);
  try
    Selected:=False;
    for i:=0 to ProbeList.Count-1 do
    begin
      Item:=lvProbes.Items.Add;
      Item.Caption:=ProbeList[i];
      Item.ImageIndex:=2;
      Item.Data:=GetProbeByName(ProbeList[i]);
      if not Selected then
      begin
        Item.Selected:=True;
        Selected:=True;
      end;
    end;
  finally
    ProbeList.Free;
  end;
end;

procedure TfmTemplateEditor.FormCreate(Sender: TObject);
begin
  FTemplate:=TSessionTemplate.create;
  frTemplate.Template:=FTemplate;
  frTemplate.TemplateToView;
  Memo1.Text:=FTemplate.Comment;
  ProbeListToView;
end;

procedure TfmTemplateEditor.FormDestroy(Sender: TObject);
begin
  if FOwnerForm<>nil then SendMessage(FOwnerForm.Handle, wm_child_closed,
    Integer(Self), 0);
  ClearProbeView;
  if not HasNoFile then FTemplate.Free;
end;

procedure TfmTemplateEditor.LoadFromFile(const FileName: string);
begin
  FTemplate.LoadFromFile(FileName);
  frTemplate.TemplateToView;
  Memo1.Text:=FTemplate.Comment;
  DataChanged:=False;
  HasNoFile:=False;
end;

procedure TfmTemplateEditor.SaveToFile(const FileName: string);
begin
  FTemplate.Comment:=Memo1.Text;
  FTemplate.SaveToFile(FileName);
  DataChanged:=False;
end;

function TfmTemplateEditor.DefaultExtension: string;
begin
  result:=extTemplate;
end;

procedure TfmTemplateEditor.lvProbesMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbLeft) then
  begin
    FDragStartX:=X;
    FDragStartY:=Y;
  end;
end;

procedure TfmTemplateEditor.lvProbesMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbLeft) then
  begin
    FDragStartX:=0;
    FDragStartY:=0;
  end;
end;

procedure TfmTemplateEditor.lvProbesMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  if FDragStartX<>0 then
  begin
    if (Abs(X-FDragStartX)>=TreshHold) or (Abs(Y-FDragStartY)>=TreshHold) then
    begin
      FDraggedNode:=frTemplate.tvTemplate.GetNodeAt(FDragStartX, FDragStartY);
      FDraggedItem:=lvProbes.GetItemAt(X, Y);
      if (FDraggedItem<>nil) then (Sender as TWinControl).BeginDrag(True);
      FDragStartX:=0;
      FDragStartY:=0;
    end;
  end;
end;

procedure TfmTemplateEditor.tvTemplateDragDrop(Sender, Source: TObject; X,
  Y: Integer);
var
  Node: TTreeNode;
  Item: TTemplateItem;
begin
  Node:=frTemplate.tvTemplate.GetNodeAt(X,Y);
  if (Node=nil) or (Node.Data=nil) then Exit;

  if (Source = lvProbes) then
  begin
    if (FDraggedItem<>nil) then
    begin
      Item:=TTemplateItem.Create(TTemplateItem(Node.Data), FDraggedItem.Caption, '');
      TTemplateItem(Node.Data).AddChild(Item);
      frTemplate.AddProbe(Node,Item).MakeVisible;
      DataChanged:=True;
    end;
  end else
  if (Source = frTemplate.tvTemplate) then
  begin
    if (FDraggedNode<>nil) then
    begin
      TTemplateItem(FDraggedNode.Data).OwnerItem.RemoveChild(TTemplateItem(FDraggedNode.Data));
      TTemplateItem(Node.Data).AddChild(TTemplateItem(FDraggedNode.Data), True);
      FDraggedNode.MoveTo(Node, naAddChildFirst);
      DataChanged:=True;
    end;
  end;
end;

procedure TfmTemplateEditor.tvTemplateDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
var
  Node: TTreeNode;
begin
  Accept:=False;
  if ((Source = lvProbes) or (Source = frTemplate.tvTemplate)) then
  begin
    Node:=frTemplate.tvTemplate.GetNodeAt(X,Y);
    if (Node<>nil) and (Node.Data<>nil) and (TObject(Node.Data) is TTemplateItem) then
    begin
      Accept:=True;
      if (Source = frTemplate.tvTemplate) and Node.HasAsParent(FDraggedNode) then Accept:=False;
    end;
  end;
end;

procedure TfmTemplateEditor.tvTemplateMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  ht: THitTests;
begin
  if (Button = mbLeft) then
  begin
    ht:=frTemplate.tvTemplate.GetHitTestInfoAt(X,Y);
    if (htOnItem in ht) or (htOnIcon in ht)  then
    begin
      FDragStartX:=X;
      FDragStartY:=Y;
    end;
  end;
end;

procedure TfmTemplateEditor.frTemplatetvTemplateMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbLeft) then
  begin
    FDragStartX:=0;
    FDragStartY:=0;
  end;
end;

procedure TfmTemplateEditor.frTemplatetvTemplateMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  if FDragStartX<>0 then
  begin
    if (Abs(X-FDragStartX)>=TreshHold) or (Abs(Y-FDragStartY)>=TreshHold) then
    begin
      FDraggedNode:=frTemplate.tvTemplate.GetNodeAt(FDragStartX, FDragStartY);
      if (FDraggedNode<>nil) and (FDraggedNode.Data<>nil) and
      (TObject(FDraggedNode.Data) is TTemplateItem) then
        (Sender as TWinControl).BeginDrag(True);
      FDragStartX:=0;
      FDragStartY:=0;
    end;
  end;
end;


procedure TfmTemplateEditor.lvProbesDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
begin
  if (Source = frTemplate.tvTemplate) and (FDraggedNode<>nil) then Accept:=True else Accept:=False;
end;

procedure TfmTemplateEditor.lvProbesDragDrop(Sender, Source: TObject; X,
  Y: Integer);
begin
  if (FDraggedNode<>nil) and (FDraggedNode.Data<>nil) then
  begin
    TTemplateItem(FDraggedNode.Data).OwnerItem.RemoveChild(TTemplateItem(FDraggedNode.Data));
    TTemplateItem(FDraggedNode.Data).Free;
    FDraggedNode.Delete;
    DataChanged:=True;
  end;
end;

procedure TfmTemplateEditor.tvTemplateEdited(Sender: TObject;
  Node: TTreeNode; var S: String);
begin
  if Node.Data<>nil then
    TTemplateItem(Node.Data).Alias:=S;
end;

procedure TfmTemplateEditor.acShowInputParamsExecute(Sender: TObject);
begin
  frTemplate.ShowInputParams:=not frTemplate.ShowInputParams;
  frTemplate.TemplateToView;
end;

procedure TfmTemplateEditor.acShowOutputParamsExecute(Sender: TObject);
begin
  frTemplate.ShowOutputParams:=not frTemplate.ShowOutputParams;
  frTemplate.TemplateToView;
end;

procedure TfmTemplateEditor.tvTemplateEditing(Sender: TObject;
  Node: TTreeNode; var AllowEdit: Boolean);
begin
  AllowEdit := (Node.Data<>nil) and (TObject(Node.Data) is TTemplateItem);
end;

procedure TfmTemplateEditor.acEditParamsExecute(Sender: TObject);
begin
  if EditParams(FTemplate, False) then DataChanged:=True;
end;

procedure TfmTemplateEditor.acEditViewExecute(Sender: TObject);
begin
  if EditView(FTemplate) then DataChanged:=True;
end;

procedure TfmTemplateEditor.SetTemplate(const Value: TSessionTemplate);
begin
  if (not HasNoFile) and (fTemplate<>nil) then fTemplate.Free;
  FTemplate := Value;
  frTemplate.Template:=Value;
  frTemplate.TemplateToView;
  Memo1.Text:=FTemplate.Comment;
  HasNoFile:=True;
end;

procedure TfmTemplateEditor.lvProbesSelectItem(Sender: TObject;
  Item: TListItem; Selected: Boolean);
begin
  if Selected and (Item<>nil) and (Item.Data<>nil) then
    Memo2.Text:=TCustomProbe(Item.Data).Description else
    Memo2.Text:='';
end;

end.
