{*******************************************************}
{          Copyright (c) 2000 Vlad Korshunov            }
{                   IPH tools project                   }
{               Template visualisation frame            }
{*******************************************************}
unit TemplateFrame;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, session, ImgList, probes, probeparams;

type
  TfrTemplate = class(TFrame)
    pnTemplate: TGroupBox;
    tvTemplate: TTreeView;
    Images: TImageList;
    procedure tvTemplateCompare(Sender: TObject; Node1, Node2: TTreeNode;
      Data: Integer; var Compare: Integer);
  private
    FShowInputParams: Boolean;
    FShowOutputParams: Boolean;
    fTemplate: TSessionTemplate;
  public
    function AddProbe(ToItem: TTreeNode;
      TemplateItem: TTemplateItem): TTreeNode;
    procedure TemplateToView;
    function IsParamNode(Node: TTreeNode): Boolean;
    property Template: TSessionTemplate read fTemplate write fTemplate;
    property ShowInputParams: Boolean read FShowInputParams write FShowInputParams;
    property ShowOutputParams: Boolean read FShowOutputParams write FShowOutputParams;
  end;

implementation
uses iphconsts;
{$R *.DFM}

function TfrTemplate.AddProbe(ToItem: TTreeNode; TemplateItem: TTemplateItem): TTreeNode;

  procedure AddParams(ToItem: TTreeNode; ParamTypes: TParamTypes);
  var
    i: integer;
    ParamNode, PrItemNode: TTreeNode;
    ParamSource: TParamSource;
  begin
    if (TemplateItem.Params.ParamTypeCount(ParamTypes)<>0) then
    begin
      ParamNode:=tvTemplate.Items.AddChild(Result, sTemplateParamNode);
      ParamNode.ImageIndex:=3;
      ParamNode.SelectedIndex:=3;
      for i:=0 to TemplateItem.Params.Count-1 do
        if TemplateItem.Params[i].ParamType in ParamTypes then
        begin
          PrItemNode:=tvTemplate.Items.AddChildObject(ParamNode,
            TemplateItem.Params[i].Name, TemplateItem.Params[i]);
          if TemplateItem.Params[i].ParamType = ptOutput then
          begin
            PrItemNode.ImageIndex:=1;
            PrItemNode.SelectedIndex:=1;
          end else
          begin
            PrItemNode.ImageIndex:=0;
            PrItemNode.SelectedIndex:=0;
            ParamSource:=TemplateItem.ParamSources.FindByParamName(TemplateItem.Params[i].Name);
            if (ParamSource<>nil) and (ParamSource.SourceParam<>nil) then
              prItemNode.Text:=prItemNode.Text+' ('+
                FTemplate.FullParamName(ParamSource.SourceParam)+')';
          end;
        end;
      end;
  end;
var
  i: integer;
  Types: TParamTypes;
begin
  Result:=tvTemplate.Items.AddChildObject(ToItem, TemplateItem.Alias, TemplateItem);
  Result.ImageIndex:=2;
  Result.SelectedIndex:=2;
  Types:=[];
  if FShowInputParams then Include(Types, ptInput);
  if FShowOutputParams then Include(Types, ptOutput);
  AddParams(Result, Types);
  for i:=0 to TemplateItem.ChildItems.Count-1 do
    AddProbe(Result, TemplateItem.ChildItems[i]);
end;

function TfrTemplate.IsParamNode(Node: TTreeNode): Boolean;
begin
  result:=(Node<>nil) and (Node.Data<>nil) and (TObject(Node.Data) is TParam);
end;

procedure TfrTemplate.TemplateToView;
begin
  tvTemplate.Items.BeginUpdate;
  try
    tvTemplate.Items.Clear;
    AddProbe(nil, FTemplate.RootItem);
    tvTemplate.Items[0].Expand(True);
    tvTemplate.CustomSort(nil, 0);
  finally
    tvTemplate.Items.EndUpdate;
  end;
end;

procedure TfrTemplate.tvTemplateCompare(Sender: TObject; Node1,
  Node2: TTreeNode; Data: Integer; var Compare: Integer);
begin
  If Node1.Data = nil then Compare:=-1
    else if Node2.Data = nil then Compare:=1 else
  begin
    if (TObject(Node1.Data) is TParam) and (TObject(Node2.Data) is TParam) then
      if (TParam(Node1.Data).ParamType = ptInput) and (TParam(Node2.Data).ParamType = ptOutput) then Compare:=-1 else
        if (TParam(Node1.Data).ParamType = ptOutput) and (TParam(Node2.Data).ParamType = ptInput) then Compare:=1 else
          Compare:=0;
  end;
end;

end.
