{*******************************************************}
{          Copyright (c) 2000 Vlad Korshunov            }
{                   IPH tools project                   }
{                 Socket utility functions              }
{*******************************************************}
unit socks;
interface
uses windows, winsock, sysutils, sockerr, math;

type
  ESocketError = class(Exception);

procedure CheckSocketLastErr(SockErr: Integer; const Op: string);
function CheckSocketResult(ResultCode: Integer; const Op: string): Integer;

procedure InitWinsock;
procedure FreeWinsock;

function GetSocket(af, Struct, protocol: Integer): Integer;
function SocketRead(Socket: integer; var B; Size: DWORD; Addr: PSockAddr= nil): integer;
function SocketReadStr(Socket: Integer; Addr: PSockAddr = nil): string;
function SocketWrite(Socket: integer; const B; Size: DWORD; Addr: PSockAddr = nil): integer;
procedure SocketConnect(Socket: integer; Addr: TSockAddr);

implementation
uses iphconsts, utils;

const
  cBufSize=$2000;

procedure CheckSocketLastErr(SockErr: Integer; const Op: string);
begin
  if SockErr<>0 then
    raise ESocketError.CreateFmt(sWindowsSocketError,
      [GetErrorDescription(SockErr), SockErr, Op]);
end;

function CheckSocketResult(ResultCode: Integer; const Op: string): Integer;
begin
  if ResultCode <> 0 then
  begin
    Result := WSAGetLastError;
    if Result=0 then RaiseLastWin32Error(OP) else
      if Result <> WSAEWOULDBLOCK then
        CheckSocketLastErr(Result, Op);
  end else Result := 0;
end;

const
  WinSockInitialized: boolean = false;

procedure InitWinsock;
var
  err: Integer;
  WData: TWSAData;
begin
  if WinSockInitialized then exit;
  Err := WSAStartup($0101, WData);
  if Err <> 0 then
    raise ESocketError.CreateFmt(sWindowsSocketError,
      [SysErrorMessage(Err), Err, 'WSAStartup']) else
    WinSockInitialized:=True;
end;

procedure FreeWinsock;
var
  Err: Integer;
begin
  if WinSockInitialized then
  begin
    Err := WSACleanup;
    if Err<>0 then
    raise ESocketError.CreateFmt(sWindowsSocketError,
      [SysErrorMessage(Err), Err, 'WSACleanup']);
  end;
end;

function GetSocket(af, Struct, protocol: Integer): Integer;
begin
  result := socket(af, Struct, protocol);
  if Result = INVALID_SOCKET then raise ESocketError.Create(sCannotCreateSocket);
end;

procedure SocketConnect(Socket: integer; Addr: TSockAddr);
begin
  CheckSocketResult(Connect(Socket, Addr, SizeOf(Addr)),'connect');
end;

procedure SocketListen(s: TSocket; backlog: Integer);
begin
  CheckSocketResult(listen(s, backlog), 'listen');
end;

function SocketRead(Socket: integer; var B; Size: DWORD; Addr: PSockAddr= nil): integer;
var
  ErrorCode: Integer;
  FromLen: Integer;
begin
  if Addr = nil then result := recv(Socket, B, Size, 0) else
  begin
    fromLen:=SizeOf(Addr^);
    Result := recvFrom(Socket, B, Size, 0, Addr^, FromLen);
  end;
  if (result = SOCKET_ERROR) or (result < 0) then
  begin
    ErrorCode := WSAGetLastError;
    raise ESocketError.CreateFmt(sWindowsSocketError,
      [SysErrorMessage(ErrorCode), ErrorCode, 'recv']);
  end;
end;

function SocketReadStr(Socket: Integer; Addr: PSockAddr = nil): string;
var
  Buf: array[0..cBufSize] of Char;
  SBuf: string;
  Size: Integer;
begin
  Result:='';
  Size:=SocketRead(Socket, Buf, SizeOf(Buf), Addr);
  SetString(SBuf, Buf, Size);
  Result:=Result+SBuf;
end;

function SocketWrite(Socket: integer; const B; Size: DWORD; Addr: PSockAddr = nil): integer;
var
  p: PByteArray;
  Written, Left, i, WriteNow: DWORD;
  ErrorCode: Integer;
begin
  p := @B;
  i := 0;
  Left := Size;
  while Left > 0 do
  begin
    WriteNow := Min(Left, cBufSize);
    if Addr = nil then
      Written := send(Socket, p^[i], WriteNow, 0) else
      Written := sendTo(Socket, p^[i], WriteNow, 0, Addr^, SizeOf(Addr^));
    if (Integer(Written) = SOCKET_ERROR) then
    begin
      ErrorCode := WSAGetLastError;
      raise ESocketError.CreateFmt(sWindowsSocketError,
        [GetErrorDescription(ErrorCode), ErrorCode, 'send']);
    end;
    Inc(i, Written);
    Dec(Left, Written);
{    if Written <> WriteNow then Break;}
  end;
  Result := i;
end;

end.
